import com.jagacy.Key;
import com.jagacy.SessionVt;
import com.jagacy.util.JagacyException;
import com.jagacy.util.JagacyProperties;
import com.jagacy.util.Loggable;

/**
 * This class doesn't use fields or positions to retrieve the first 5 java
 * titles from the Boston University library. Not as robust as the other
 * examples.
 * 
 * @author Robert M. Preston
 * 
 */
public class Example1 extends SessionVt {

    private JagacyProperties props;

    private Loggable logger;

    private Example1() throws JagacyException {
        super("example1");
        props = getProperties();
        logger = getLoggable();
    }

    /* (non-Javadoc)
     * @see com.jagacy.AbstractSession#logon()
     */
    protected boolean logon() throws JagacyException {
        // Notice that you don't have to prefix each property with 'example1'.
        // Jagacy will do this for you. But, you do have to prefix the
        // entries in the property file.
        waitForChange("logon.mainTimeout.seconds", "logon.trailingTimeout",
            "logon.tag");
        boolean success = readRow(2).indexOf(props.get("logon.id")) != -1;
        if (!success) {
            logger.fatal("Not main screen");
        }

        return success;
    }

    /* (non-Javadoc)
     * @see com.jagacy.AbstractSession#logoff()
     */
    protected void logoff() throws JagacyException {
        writeKey(Key.ENTER);
        writeKey(Key.ENTER);
        waitForChange("logoff.mainTimeout.seconds", "logoff.trailingTimeout");
        
        writeProperty("logoff1.entry");
        writeKey(Key.ENTER);
        waitForChange("logoff.mainTimeout.seconds", "logoff.trailingTimeout",
            "logoff1.tag");

        if (readRow(0).indexOf(props.get("menu.id")) != -1) {
            writeProperty("logoff2.entry");
        }
    }

    /**
     * Retrieves the first 5 Java titles and writes them to System.out.
     * 
     * @throws JagacyException If an error occurs.
     */
    private void getJavaTitles() throws JagacyException {
        writeAfterLabel("main");
        writeKey(Key.ENTER);

        waitForChange("menu.mainTimeout.seconds", "menu.trailingTimeout",
            "menu.tag");
        if (readRow(0).indexOf(props.get("menu.id")) == -1) {
            logger.fatal("Not menu screen");
            return;
        }

        writeProperty("menu.entry");
        writeKey(Key.ENTER);

        waitForChange("keyword.mainTimeout.seconds", "keyword.trailingTimeout",
            "keyword.tag");
        if (readRow(21).indexOf(props.get("keyword.id")) == -1) {
            logger.fatal("Not keyword screen");
            return;
        }

        writeString("java");
        writeKey(Key.ENTER);

        do {
            waitForChange("working.mainTimeout.seconds",
                "working.trailingTimeout");
        } while (readRow(0).indexOf(props.get("working.tag")) != -1);

        writeKey(Key.ENTER);

        waitForChange("data.mainTimeout.seconds", "data.trailingTimeout",
            "data.tag");
        if (readRow(23).indexOf(props.get("data.id")) == -1) {
            logger.fatal("Not data screen");
            return;
        }

        String[] screen = readScreen();
        for (int i = 2; i < screen.length; i++) {
            if (screen[i].indexOf(props.get("data.end.id")) != -1) {
                break;
            }

            System.out.println(screen[i]);
        }
    }

    /**
     * Print exception(s) to System.err.
     * 
     * @param e JagacyException
     */
    private static void printExceptions(JagacyException e) {
        System.err.println(e);
        if (e.hasException()) {
            System.err.println(e.getException());
        }
    }

    public static void main(String[] args) {
        Example1 example = null;
        try {
            example = new Example1();
            example.open();
            example.getJavaTitles();
            example.close();
        } catch (JagacyException e) {
            printExceptions(e);
            if (example != null) {
                example.abort();
            }
        }

        if ((example != null) && example.props.getBoolean("window", false)) {
            // Swing requires this if window is enabled.
            System.exit(0);
        }
    }
}