import com.jagacy.Key;
import com.jagacy.SessionVt;
import com.jagacy.util.JagacyException;
import com.jagacy.util.JagacyProperties;
import com.jagacy.util.Loggable;

/**
 * This class uses positions to retrieve the first 5 java titles from the Boston
 * University library.
 * 
 * @author Robert M. Preston
 * 
 */
public class Example2 extends SessionVt {

    private JagacyProperties props;

    private Loggable logger;

    private Example2() throws JagacyException {
        super("example2");
        props = getProperties();
        logger = getLoggable();
    }

    /* (non-Javadoc)
     * @see com.jagacy.AbstractSession#logon()
     */
    protected boolean logon() throws JagacyException {
        // Notice that you don't have to prefix each property with 'example2'.
        // Jagacy will do this for you. But, you do have to prefix the
        // entries in the property file.
        boolean success = waitForPositionProperty("logon.wait",
            "logon.mainTimeout.seconds", "logon.trailingTimeout");
        if (!success) {
            logger.fatal("Not main screen");
        }

        return success;
    }

    /* (non-Javadoc)
     * @see com.jagacy.AbstractSession#logoff()
     */
    protected void logoff() throws JagacyException {
        writeKey(Key.ENTER);
        writeKey(Key.ENTER);
        waitForChange("logoff.mainTimeout.seconds", "logoff.trailingTimeout");

        writePosition("logoff.entry");
        writeKey(Key.ENTER);

        if (waitForPositionProperty("menu.wait", "logoff.mainTimeout.seconds",
            "logoff.trailingTimeout")) {
            writePosition("menu.entry", props.get("menu.entry.endValue"));
        }
    }

    /**
     * Retrieves the first 5 Java titles and writes them to System.out.
     * 
     * @throws JagacyException If an error occurs.
     */
    private void getJavaTitles() throws JagacyException {
        writePosition("main.entry");
        writeKey(Key.ENTER);

        if (!waitForPositionProperty("menu.wait", "menu.mainTimeout.seconds",
            "menu.trailingTimeout")) {
            logger.fatal("Not menu screen");
            return;
        }

        writePosition("menu.entry");
        writeKey(Key.ENTER);

        if (!waitForPositionProperty("keyword.wait",
            "keyword.mainTimeout.seconds", "keyword.trailingTimeout")) {
            logger.fatal("Not keyword screen");
            return;
        }

        writePosition("keyword.entry", "java");
        writeKey(Key.ENTER);

        if (!waitForPositionProperty("data.wait", "data.mainTimeout.seconds",
            "data.trailingTimeout")) {
            logger.fatal("Not data screen");
            return;
        }

        for (int i = props.getCardinal("data.begin"), length = props
            .getCardinal("data.end"); i < length; i++) {
            System.out.println(readPosition(i, 0, getWidth()));
        }
    }

    /**
     * Print exception(s) to System.err.
     * 
     * @param e JagacyException
     */
    private static void printExceptions(JagacyException e) {
        System.err.println(e);
        if (e.hasException()) {
            System.err.println(e.getException());
        }
    }

    public static void main(String[] args) {
        Example2 example = null;
        try {
            example = new Example2();
            example.open();
            example.getJavaTitles();
            example.close();
        } catch (JagacyException e) {
            printExceptions(e);
            if (example != null) {
                example.abort();
            }
        }

        if ((example != null) && example.props.getBoolean("window", false)) {
            // Swing requires this if window is enabled.
            System.exit(0);
        }
    }
}