import com.jagacy.SessionVt;
import com.jagacy.util.JagacyException;
import com.jagacy.util.JagacyProperties;

/**
 * This class is unique. Each method is a simple state machine totally driven by
 * the properties file. It never has to be changed or recompiled! Its better
 * than a macro language.
 * 
 * @author Robert M. Preston
 * 
 */
public class Example3 extends SessionVt {

    private JagacyProperties props;

    private Example3() throws JagacyException {
        super("example3");
        props = getProperties();
    }

    /* (non-Javadoc)
     * @see com.jagacy.AbstractSession#logon()
     */
    protected boolean logon() throws JagacyException {
        // Notice that you don't have to prefix each property with 'example3'.
        // Jagacy will do this for you. But, you do have to prefix the
        // entries in the property file.
        boolean isWait = true;
        for (String state = "Begin"; true;) {
            if (isWait) {
                waitForChange("logon.mainTimeout." + state + ".seconds",
                    "logon.trailingTimeout." + state, "logon.tag." + state);
            }
            if (!waitForPosition("logon.position." + state, 0)) {
                state = props.get("logon.nextState.alt." + state);
                continue;
            }
            if (props.getBoolean("logon.done." + state, false)) {
                break;
            }
            if (props.getBoolean("logon.entry." + state, true)) {
                writePosition("logon.entry." + state);
            }

            if (props.getBoolean("logon.keyEntry." + state, true)) {
                isWait = writeKey("logon.key." + state).isWaitKey();
            } else {
                isWait = true;
            }
            state = props.get("logon.nextState." + state);
        }

        return true;
    }

    /* (non-Javadoc)
     * @see com.jagacy.AbstractSession#logoff()
     */
    protected void logoff() throws JagacyException {
        boolean isWait = true;
        for (String state = "Begin"; true;) {
            if (isWait) {
                waitForChange("logoff.mainTimeout." + state + ".seconds",
                    "logoff.trailingTimeout." + state, "logoff.tag." + state);
            }
            if (props.getBoolean("logoff.done." + state, false)) {
                break;
            }
            if (!waitForPosition("logoff.position." + state, 0)) {
                state = props.get("logoff.nextState.alt." + state);
                continue;
            }
            if (props.getBoolean("logoff.entry." + state, true)) {
                writePosition("logoff.entry." + state);
            }
            if (props.getBoolean("logoff.keyEntry." + state, true)) {
                isWait = writeKey("logoff.key." + state).isWaitKey();
            } else {
                isWait = true;
            }
            state = props.get("logoff.nextState." + state);
        }
    }

    /**
     * Retrieves the first 5 Java titles and writes them to System.out.
     * 
     * @throws JagacyException If an error occurs.
     */
    private void getJavaTitles() throws JagacyException {
        boolean isWait = true;
        for (String state = "Begin"; true;) {
            if (isWait) {
                waitForChange("data.mainTimeout." + state + ".seconds",
                    "data.trailingTimeout." + state, "data.tag." + state);
            }
            if (!waitForPosition("data.position." + state, 0)) {
                state = props.get("data.nextState.alt." + state);
                continue;
            }
            if (props.getBoolean("data.done." + state, false)) {
                break;
            }
            if (props.getBoolean("data.entry." + state, true)) {
                writePosition("data.entry." + state);
            }
            if (props.getBoolean("data.keyEntry." + state, true)) {
                isWait = writeKey("data.key." + state).isWaitKey();
            } else {
                isWait = true;
            }
            state = props.get("data.nextState." + state);
        }

        for (int i = props.getCardinal("data.begin"), length = props
            .getCardinal("data.end"); i < length; i++) {

            System.out.println(readPosition(i, 0, getWidth()));
        }
    }

    /**
     * Print exception(s) to System.err.
     * 
     * @param e JagacyException
     */
    private static void printExceptions(JagacyException e) {
        System.err.println(e);
        if (e.hasException()) {
            System.err.println(e.getException());
        }
    }

    public static void main(String[] args) {
        Example3 example = null;
        try {
            example = new Example3();
            example.open();
            example.getJavaTitles();
            example.close();
        } catch (JagacyException e) {
            printExceptions(e);
            if (example != null) {
                example.abort();
            }
        }

        if ((example != null) && example.props.getBoolean("window", false)) {
            // Swing requires this if window is enabled.
            System.exit(0);
        }
    }
}